@@ -40,14 +40,12 @@ openstack catalog list
 
 export GNOCCHI_SERVICE_TOKEN=$(openstack token issue -c id -f value)
 export GNOCCHI_SERVICE_URL=$(openstack catalog show metric -c endpoints -f value | awk '/public/{print $2}')
-export GNOCCHI_AUTHORIZATION=""  # Temporary set to transition to the new functional testing
 
 curl -X GET ${GNOCCHI_SERVICE_URL}/v1/archive_policy -H "Content-Type: application/json"
 
 sudo gnocchi-upgrade
 
 # Just ensure tools still works
-gnocchi metric create
 sudo -E -H -u stack $GNOCCHI_DIR/tools/measures_injector.py --metrics 1 --batch-of-measures 2 --measures-per-batch 2
 
 # NOTE(sileht): on swift job permissions are wrong, I don't known why
