@@ -121,7 +121,7 @@ function install_gnocchiclient {
         git_clone_by_name python-gnocchiclient
         setup_dev_lib python-gnocchiclient
     else
-        pip_install 'gnocchiclient<3.0'
+        pip_install gnocchiclient
     fi
 }
 
@@ -228,10 +228,6 @@ function configure_gnocchi {
     # Configure logging
     iniset $GNOCCHI_CONF DEFAULT debug "$ENABLE_DEBUG_LOG_LEVEL"
 
-    # Install the configuration files
-    cp $GNOCCHI_DIR/etc/gnocchi/* $GNOCCHI_CONF_DIR
-
-
     # Set up logging
     if [ "$SYSLOG" != "False" ]; then
         iniset $GNOCCHI_CONF DEFAULT use_syslog "True"
@@ -246,9 +242,6 @@ function configure_gnocchi {
         iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
     fi
 
-    # Configure auth token middleware
-    configure_auth_token_middleware $GNOCCHI_CONF gnocchi $GNOCCHI_AUTH_CACHE_DIR
-
     if is_service_enabled gnocchi-statsd ; then
         iniset $GNOCCHI_CONF statsd resource_id $GNOCCHI_STATSD_RESOURCE_ID
         iniset $GNOCCHI_CONF statsd project_id $GNOCCHI_STATSD_PROJECT_ID
@@ -276,12 +269,14 @@ function configure_gnocchi {
     fi
 
     if [ "$GNOCCHI_USE_KEYSTONE" == "True" ] ; then
-        iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi+auth
+        # Configure auth token middleware
+        configure_auth_token_middleware $GNOCCHI_CONF gnocchi $GNOCCHI_AUTH_CACHE_DIR
+        iniset $GNOCCHI_CONF api auth_mode keystone
         if is_service_enabled gnocchi-grafana; then
             iniset $GNOCCHI_CONF cors allowed_origin ${GRAFANA_URL}
         fi
     else
-        iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi+noauth
+        inidelete $GNOCCHI_CONF api auth_mode
     fi
 
     # Configure the indexer database
@@ -437,14 +432,6 @@ function start_gnocchi {
         fi
     fi
 
-    # Create a default policy
-    if [ "$GNOCCHI_USE_KEYSTONE" == "False" ]; then
-        export OS_AUTH_TYPE=gnocchi-noauth
-        export GNOCCHI_USER_ID=`uuidgen`
-        export GNOCCHI_PROJECT_ID=`uuidgen`
-        export GNOCCHI_ENDPOINT="$(gnocchi_service_url)"
-    fi
-
     # run metricd last so we are properly waiting for swift and friends
     run_process gnocchi-metricd "$GNOCCHI_BIN_DIR/gnocchi-metricd -d -v --config-file $GNOCCHI_CONF"
     run_process gnocchi-statsd "$GNOCCHI_BIN_DIR/gnocchi-statsd -d -v --config-file $GNOCCHI_CONF"
@@ -472,12 +459,12 @@ if is_service_enabled gnocchi-api; then
         configure_keystone_for_gnocchi
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         echo_summary "Configuring Gnocchi"
-        configure_gnocchi
-        create_gnocchi_accounts
         if _is_ceph_enabled && [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
             echo_summary "Configuring Gnocchi for Ceph"
             configure_ceph_gnocchi
         fi
+        configure_gnocchi
+        create_gnocchi_accounts
     elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
         echo_summary "Initializing Gnocchi"
         init_gnocchi
