@@ -18,7 +18,6 @@
 
 import datetime
 import functools
-import itertools
 import logging
 import math
 import numbers
@@ -28,7 +27,10 @@ import struct
 import time
 
 import lz4
+import numpy
+import numpy.lib.recfunctions
 import pandas
+from scipy import ndimage
 import six
 
 # NOTE(sileht): pandas relies on time.strptime()
@@ -74,11 +76,96 @@ class UnknownAggregationMethod(Exception):
             "Unknown aggregation method `%s'" % agg)
 
 
+class InvalidData(ValueError):
+    """Error raised when data are corrupted."""
+    def __init__(self):
+        super(InvalidData, self).__init__("Unable to unpack, invalid data")
+
+
 def round_timestamp(ts, freq):
     return pandas.Timestamp(
         (pandas.Timestamp(ts).value // freq) * freq)
 
 
+class GroupedTimeSeries(object):
+    def __init__(self, ts, granularity):
+        # NOTE(sileht): The whole class assumes ts is ordered and don't have
+        # duplicate timestamps, it uses numpy.unique that sorted list, but
+        # we always assume the orderd to be the same as the input.
+        freq = granularity * 10e8
+        self._ts = ts
+        self.indexes = (numpy.array(ts.index, 'float') // freq) * freq
+        self.tstamps, self.counts = numpy.unique(self.indexes,
+                                                 return_counts=True)
+
+    def mean(self):
+        return self._scipy_aggregate(ndimage.mean)
+
+    def sum(self):
+        return self._scipy_aggregate(ndimage.sum)
+
+    def min(self):
+        return self._scipy_aggregate(ndimage.minimum)
+
+    def max(self):
+        return self._scipy_aggregate(ndimage.maximum)
+
+    def median(self):
+        return self._scipy_aggregate(ndimage.median)
+
+    def std(self):
+        # NOTE(sileht): ndimage.standard_deviation is really more performant
+        # but it use ddof=0, to get the same result as pandas we have to use
+        # ddof=1. If one day scipy allow to pass ddof, this should be changed.
+        return self._scipy_aggregate(ndimage.labeled_comprehension,
+                                     remove_unique=True,
+                                     func=functools.partial(numpy.std, ddof=1),
+                                     out_dtype='float64',
+                                     default=None)
+
+    def _count(self):
+        timestamps = numpy.array(self.tstamps, 'datetime64[ns]')
+        return (self.counts, timestamps)
+
+    def count(self):
+        return pandas.Series(*self._count())
+
+    def last(self):
+        counts, timestamps = self._count()
+        cumcounts = numpy.cumsum(counts) - 1
+        values = self._ts.values[cumcounts]
+        return pandas.Series(values, pandas.to_datetime(timestamps))
+
+    def first(self):
+        counts, timestamps = self._count()
+        counts = numpy.insert(counts[:-1], 0, 0)
+        cumcounts = numpy.cumsum(counts)
+        values = self._ts.values[cumcounts]
+        return pandas.Series(values, pandas.to_datetime(timestamps))
+
+    def quantile(self, q):
+        return self._scipy_aggregate(ndimage.labeled_comprehension,
+                                     func=functools.partial(
+                                         numpy.percentile,
+                                         q=q,
+                                     ),
+                                     out_dtype='float64',
+                                     default=None)
+
+    def _scipy_aggregate(self, method, remove_unique=False, *args, **kwargs):
+        if remove_unique:
+            locs = numpy.argwhere(self.counts > 1).T[0]
+
+        values = method(self._ts.values, self.indexes, self.tstamps,
+                        *args, **kwargs)
+        timestamps = numpy.array(self.tstamps, 'datetime64[ns]')
+
+        if remove_unique:
+            timestamps = timestamps[locs]
+            values = values[locs]
+        return pandas.Series(values, pandas.to_datetime(timestamps))
+
+
 class TimeSerie(object):
     """A representation of series of a timestamp with a value.
 
@@ -89,7 +176,7 @@ class TimeSerie(object):
     def __init__(self, ts=None):
         if ts is None:
             ts = pandas.Series()
-        self.ts = self.clean_ts(ts)
+        self.ts = ts
 
     @staticmethod
     def clean_ts(ts):
@@ -100,8 +187,12 @@ class TimeSerie(object):
         return ts
 
     @classmethod
-    def from_data(cls, timestamps=None, values=None):
-        return cls(pandas.Series(values, timestamps))
+    def from_data(cls, timestamps=None, values=None, clean=False):
+        ts = pandas.Series(values, timestamps)
+        if clean:
+            # For format v2
+            ts = cls.clean_ts(ts)
+        return cls(ts)
 
     @classmethod
     def from_tuples(cls, timestamps_values):
@@ -123,7 +214,8 @@ class TimeSerie(object):
 
     @staticmethod
     def _timestamps_and_values_from_dict(values):
-        timestamps = pandas.to_datetime(list(values.keys()), unit='ns')
+        timestamps = numpy.array(list(values.keys()), dtype='datetime64[ns]')
+        timestamps = pandas.to_datetime(timestamps)
         v = list(values.values())
         if v:
             return timestamps, v
@@ -149,14 +241,14 @@ class TimeSerie(object):
         except IndexError:
             return
 
-    def group_serie(self, granularity, start=None):
+    def group_serie(self, granularity, start=0):
         # NOTE(jd) Our whole serialization system is based on Epoch, and we
         # store unsigned integer, so we can't store anything before Epoch.
         # Sorry!
         if self.ts.index[0].value < 0:
             raise BeforeEpochError(self.ts.index[0])
-        return self.ts[start:].groupby(functools.partial(
-            round_timestamp, freq=granularity * 10e8))
+
+        return GroupedTimeSeries(self.ts[start:], granularity)
 
 
 class BoundTimeSerie(TimeSerie):
@@ -218,6 +310,7 @@ class BoundTimeSerie(TimeSerie):
         self._truncate()
 
     _SERIALIZATION_TIMESTAMP_VALUE_LEN = struct.calcsize("<Qd")
+    _SERIALIZATION_TIMESTAMP_LEN = struct.calcsize("<Q")
 
     @classmethod
     def unserialize(cls, data, block_size, back_window):
@@ -225,32 +318,29 @@ class BoundTimeSerie(TimeSerie):
         nb_points = (
             len(uncompressed) // cls._SERIALIZATION_TIMESTAMP_VALUE_LEN
         )
-        deserial = struct.unpack("<" + "Q" * nb_points + "d" * nb_points,
-                                 uncompressed)
-        start = deserial[0]
-        timestamps = [start]
-        for delta in itertools.islice(deserial, 1, nb_points):
-            ts = start + delta
-            timestamps.append(ts)
-            start = ts
+        timestamps_raw = uncompressed[
+            :nb_points*cls._SERIALIZATION_TIMESTAMP_LEN]
+        timestamps = numpy.frombuffer(timestamps_raw, dtype='<Q')
+        timestamps = numpy.cumsum(timestamps)
+        timestamps = numpy.array(timestamps, dtype='datetime64[ns]')
+
+        values_raw = uncompressed[nb_points*cls._SERIALIZATION_TIMESTAMP_LEN:]
+        values = numpy.frombuffer(values_raw, dtype='<d')
+
         return cls.from_data(
-            pandas.to_datetime(timestamps, unit='ns'),
-            deserial[nb_points:],
+            pandas.to_datetime(timestamps),
+            values,
             block_size=block_size,
             back_window=back_window)
 
     def serialize(self):
         # NOTE(jd) Use a double delta encoding for timestamps
-        timestamps = [self.first.value]
-        start = self.first.value
-        for i in self.ts.index[1:]:
-            v = i.value
-            timestamps.append(v - start)
-            start = v
-        values = self.ts.values.tolist()
-        return lz4.dumps(struct.pack(
-            '<' + 'Q' * len(timestamps) + 'd' * len(values),
-            *(timestamps + values)))
+        timestamps = numpy.insert(numpy.diff(self.ts.index),
+                                  0, self.first.value)
+        timestamps = numpy.array(timestamps, dtype='<Q')
+        values = numpy.array(self.ts.values, dtype='<d')
+        payload = (timestamps.tobytes() + values.tobytes())
+        return lz4.dumps(payload)
 
     @classmethod
     def benchmark(cls):
@@ -288,6 +378,7 @@ class BoundTimeSerie(TimeSerie):
                                     seconds=i * random.randint(1, 10),
                                     microseconds=random.randint(1, 999999))
                                  for i in six.moves.range(points)])
+            pts = pts.sort_index()
             ts = cls(ts=pts)
             t0 = time.time()
             for i in six.moves.range(serialize_times):
@@ -403,9 +494,9 @@ class AggregatedTimeSerie(TimeSerie):
 
     PADDED_SERIAL_LEN = struct.calcsize("<?d")
     COMPRESSED_SERIAL_LEN = struct.calcsize("<Hd")
+    COMPRESSED_TIMESPAMP_LEN = struct.calcsize("<H")
 
-    def __init__(self, sampling, aggregation_method,
-                 ts=None, max_size=None):
+    def __init__(self, sampling, aggregation_method, ts=None, max_size=None):
         """A time serie that is downsampled.
 
         Used to represent the downsampled timeserie for a single
@@ -413,15 +504,15 @@ class AggregatedTimeSerie(TimeSerie):
 
         """
         super(AggregatedTimeSerie, self).__init__(ts)
-
-        self.aggregation_method_func_name, self.q = self._get_agg_method(
-            aggregation_method)
-
         self.sampling = self._to_offset(sampling).nanos / 10e8
         self.max_size = max_size
         self.aggregation_method = aggregation_method
         self._truncate(quick=True)
 
+    def resample(self, sampling):
+        return AggregatedTimeSerie.from_grouped_serie(
+            self.group_serie(sampling), sampling, self.aggregation_method)
+
     @classmethod
     def from_data(cls, sampling, aggregation_method, timestamps=None,
                   values=None, max_size=None):
@@ -435,22 +526,32 @@ class AggregatedTimeSerie(TimeSerie):
         q = None
         m = AggregatedTimeSerie._AGG_METHOD_PCT_RE.match(aggregation_method)
         if m:
-            q = float(m.group(1)) / 100
+            q = float(m.group(1))
             aggregation_method_func_name = 'quantile'
         else:
-            if not hasattr(pandas.core.groupby.SeriesGroupBy,
-                           aggregation_method):
+            if not hasattr(GroupedTimeSeries, aggregation_method):
                 raise UnknownAggregationMethod(aggregation_method)
             aggregation_method_func_name = aggregation_method
         return aggregation_method_func_name, q
 
     def split(self):
-        groupby = self.ts.groupby(functools.partial(
-            SplitKey.from_timestamp_and_sampling, sampling=self.sampling))
-        for group, ts in groupby:
-            yield (SplitKey(group, self.sampling),
+        # NOTE(sileht): We previously use groupby with
+        # SplitKey.from_timestamp_and_sampling, but
+        # this is slow because pandas can do that on any kind DataFrame
+        # but we have ordered timestamps, so don't need
+        # to iter the whole series.
+        freq = self.sampling * SplitKey.POINTS_PER_SPLIT
+        ix = numpy.array(self.ts.index, 'float64') / 10e8
+        keys, counts = numpy.unique((ix // freq) * freq, return_counts=True)
+        start = 0
+        for key, count in six.moves.zip(keys, counts):
+            end = start + count
+            if key == -0.0:
+                key = abs(key)
+            yield (SplitKey(key, self.sampling),
                    AggregatedTimeSerie(self.sampling, self.aggregation_method,
-                                       ts))
+                                       self.ts[start:end]))
+            start = end
 
     @classmethod
     def from_timeseries(cls, timeseries, sampling, aggregation_method,
@@ -468,7 +569,7 @@ class AggregatedTimeSerie(TimeSerie):
         agg_name, q = cls._get_agg_method(aggregation_method)
         return cls(sampling, aggregation_method,
                    ts=cls._resample_grouped(grouped_serie, agg_name,
-                                            q).dropna(),
+                                            q),
                    max_size=max_size)
 
     def __eq__(self, other):
@@ -495,35 +596,40 @@ class AggregatedTimeSerie(TimeSerie):
     @classmethod
     def unserialize(cls, data, start, agg_method, sampling):
         x, y = [], []
+
         start = float(start)
         if data:
             if cls.is_compressed(data):
                 # Compressed format
                 uncompressed = lz4.loads(memoryview(data)[1:].tobytes())
                 nb_points = len(uncompressed) // cls.COMPRESSED_SERIAL_LEN
-                deserial = struct.unpack(
-                    '<' + 'H' * nb_points + 'd' * nb_points,
-                    uncompressed)
-                for delta in itertools.islice(deserial, nb_points):
-                    ts = start + (delta * sampling)
-                    y.append(ts)
-                    start = ts
-                x = deserial[nb_points:]
+
+                timestamps_raw = uncompressed[
+                    :nb_points*cls.COMPRESSED_TIMESPAMP_LEN]
+                try:
+                    y = numpy.frombuffer(timestamps_raw, dtype='<H')
+                except ValueError:
+                    raise InvalidData()
+                y = numpy.cumsum(y * sampling) + start
+
+                values_raw = uncompressed[
+                    nb_points*cls.COMPRESSED_TIMESPAMP_LEN:]
+                x = numpy.frombuffer(values_raw, dtype='<d')
+
             else:
                 # Padded format
-                nb_points = len(data) // cls.PADDED_SERIAL_LEN
-                # NOTE(gordc): use '<' for standardized
-                # little-endian byte order
-                deserial = struct.unpack('<' + '?d' * nb_points, data)
-                # alternating split into 2 list and drop items with False flag
-                for i, val in itertools.compress(
-                        six.moves.zip(six.moves.range(nb_points),
-                                      deserial[1::2]),
-                        deserial[::2]):
-                    x.append(val)
-                    y.append(start + (i * sampling))
+                try:
+                    everything = numpy.frombuffer(data, dtype=[('b', '<?'),
+                                                               ('v', '<d')])
+                except ValueError:
+                    raise InvalidData()
+                index = numpy.nonzero(everything['b'])[0]
+                y = index * sampling + start
+                x = everything['v'][index]
 
-            y = pandas.to_datetime(y, unit='s')
+            y = numpy.array(y, dtype='float64') * 10e8
+            y = numpy.array(y, dtype='datetime64[ns]')
+            y = pandas.to_datetime(y)
         return cls.from_data(sampling, agg_method, y, x)
 
     def get_split_key(self, timestamp=None):
@@ -564,15 +670,13 @@ class AggregatedTimeSerie(TimeSerie):
         # initialize list to store alternating delimiter, float entries
         if compressed:
             # NOTE(jd) Use a double delta encoding for timestamps
-            timestamps = []
-            for i in self.ts.index:
-                v = i.value
-                timestamps.append(int((v - start) // offset_div))
-                start = v
-            values = self.ts.values.tolist()
-            return None, b"c" + lz4.dumps(struct.pack(
-                '<' + 'H' * len(timestamps) + 'd' * len(values),
-                *(timestamps + values)))
+            timestamps = numpy.insert(
+                numpy.diff(self.ts.index) // offset_div,
+                0, int((self.first.value - start) // offset_div))
+            timestamps = numpy.array(timestamps, dtype='<H')
+            values = numpy.array(self.ts.values, dtype='<d')
+            payload = (timestamps.tobytes() + values.tobytes())
+            return None, b"c" + lz4.dumps(payload)
         # NOTE(gordc): this binary serializes series based on the split
         # time. the format is 1B True/False flag which denotes whether
         # subsequent 8B is a real float or zero padding. every 9B
@@ -583,16 +687,27 @@ class AggregatedTimeSerie(TimeSerie):
         # series runs until and initialize list to store alternating
         # delimiter, float entries
         first = self.first.value  # NOTE(jd) needed because faster
-        e_offset = int(
-            (self.last.value - first) // offset_div) + 1
-        serial = [False] * e_offset * 2
-        for i, v in self.ts.iteritems():
-            # overwrite zero padding with real points and set flag True
-            loc = int((i.value - first) // offset_div) * 2
-            serial[loc] = True
-            serial[loc + 1] = v
+        e_offset = int((self.last.value - first) // offset_div) + 1
+
+        locs = (numpy.cumsum(numpy.diff(self.ts.index)) // offset_div)
+        locs = numpy.insert(locs, 0, 0)
+        locs = numpy.array(locs, dtype='int')
+
+        # Fill everything with zero
+        serial_dtype = [('b', '<?'), ('v', '<d')]
+        serial = numpy.zeros((e_offset,), dtype=serial_dtype)
+
+        # Create a structured array with two dimensions
+        values = numpy.array(self.ts.values, dtype='<d')
+        ones = numpy.ones_like(values, dtype='<?')
+        values = numpy.core.records.fromarrays((ones, values),
+                                               dtype=serial_dtype)
+
+        serial[locs] = values
+
+        payload = serial.tobytes()
         offset = int((first - start) // offset_div) * self.PADDED_SERIAL_LEN
-        return offset, struct.pack('<' + '?d' * e_offset, *serial)
+        return offset, payload
 
     def _truncate(self, quick=False):
         """Truncate the timeserie."""
@@ -641,7 +756,7 @@ class AggregatedTimeSerie(TimeSerie):
         """Run a speed benchmark!"""
         points = SplitKey.POINTS_PER_SPLIT
         sampling = 5
-        serialize_times = 50
+        resample = 35
 
         now = datetime.datetime(2015, 4, 3, 23, 11)
 
@@ -668,9 +783,11 @@ class AggregatedTimeSerie(TimeSerie):
                              for x in six.moves.range(points)]),
         ]:
             print(title)
+            serialize_times = 50
             pts = pandas.Series(values,
                                 [now + datetime.timedelta(seconds=i*sampling)
                                  for i in six.moves.range(points)])
+            pts = pts.sort_index()
             ts = cls(ts=pts, sampling=sampling, aggregation_method='mean')
             t0 = time.time()
             key = ts.get_split_key()
@@ -707,9 +824,39 @@ class AggregatedTimeSerie(TimeSerie):
                   % (((points * 2 * 8)
                       / ((t1 - t0) / serialize_times)) / (1024.0 * 1024.0)))
 
+            t0 = time.time()
+            for i in six.moves.range(serialize_times):
+                list(ts.split())
+            t1 = time.time()
+            print("  split() speed: %.8f s" % ((t1 - t0) / serialize_times))
+
+            # NOTE(sileht): propose a new series with half overload timestamps
+            pts = ts.ts.copy(deep=True)
+            tsbis = cls(ts=pts, sampling=sampling, aggregation_method='mean')
+            tsbis.ts.reindex(tsbis.ts.index -
+                             datetime.timedelta(seconds=sampling * points / 2))
+
+            t0 = time.time()
+            for i in six.moves.range(serialize_times):
+                ts.merge(tsbis)
+            t1 = time.time()
+            print("  merge() speed: %.8f s" % ((t1 - t0) / serialize_times))
+
+            for agg in ['mean', 'sum', 'max', 'min', 'std', 'median', 'first',
+                        'last', 'count', '5pct', '90pct']:
+                serialize_times = 3 if agg.endswith('pct') else 10
+                ts = cls(ts=pts, sampling=sampling, aggregation_method=agg)
+                t0 = time.time()
+                for i in six.moves.range(serialize_times):
+                    ts.resample(resample)
+                t1 = time.time()
+                print("  resample(%s) speed: %.8f s" % (agg, (t1 - t0) /
+                                                        serialize_times))
+
     @staticmethod
     def aggregated(timeseries, aggregation, from_timestamp=None,
-                   to_timestamp=None, needed_percent_of_overlap=100.0):
+                   to_timestamp=None, needed_percent_of_overlap=100.0,
+                   fill=None):
 
         index = ['timestamp', 'granularity']
         columns = ['timestamp', 'granularity', 'value']
@@ -733,57 +880,65 @@ class AggregatedTimeSerie(TimeSerie):
             set(ts.sampling for ts in timeseries)
         )
 
-        grouped = pandas.concat(dataframes).groupby(level=index)
         left_boundary_ts = None
         right_boundary_ts = None
-        maybe_next_timestamp_is_left_boundary = False
+        if fill is not None:
+            fill_df = pandas.concat(dataframes, axis=1)
+            if fill != 'null':
+                fill_df = fill_df.fillna(fill)
+            single_df = pandas.concat([series for __, series in
+                                       fill_df.iteritems()]).to_frame()
+            grouped = single_df.groupby(level=index)
+        else:
+            grouped = pandas.concat(dataframes).groupby(level=index)
+            maybe_next_timestamp_is_left_boundary = False
 
-        left_holes = 0
-        right_holes = 0
-        holes = 0
-        for (timestamp, __), group in grouped:
-            if group.count()['value'] != number_of_distinct_datasource:
-                maybe_next_timestamp_is_left_boundary = True
-                if left_boundary_ts is not None:
-                    right_holes += 1
+            left_holes = 0
+            right_holes = 0
+            holes = 0
+            for (timestamp, __), group in grouped:
+                if group.count()['value'] != number_of_distinct_datasource:
+                    maybe_next_timestamp_is_left_boundary = True
+                    if left_boundary_ts is not None:
+                        right_holes += 1
+                    else:
+                        left_holes += 1
+                elif maybe_next_timestamp_is_left_boundary:
+                    left_boundary_ts = timestamp
+                    maybe_next_timestamp_is_left_boundary = False
                 else:
-                    left_holes += 1
-            elif maybe_next_timestamp_is_left_boundary:
-                left_boundary_ts = timestamp
-                maybe_next_timestamp_is_left_boundary = False
-            else:
-                right_boundary_ts = timestamp
+                    right_boundary_ts = timestamp
+                    holes += right_holes
+                    right_holes = 0
+
+            if to_timestamp is not None:
+                holes += left_holes
+            if from_timestamp is not None:
                 holes += right_holes
-                right_holes = 0
 
-        if to_timestamp is not None:
-            holes += left_holes
-        if from_timestamp is not None:
-            holes += right_holes
-
-        if to_timestamp is not None or from_timestamp is not None:
-            maximum = len(grouped)
-            percent_of_overlap = (float(maximum - holes) * 100.0 /
-                                  float(maximum))
-            if percent_of_overlap < needed_percent_of_overlap:
-                raise UnAggregableTimeseries(
-                    'Less than %f%% of datapoints overlap in this '
-                    'timespan (%.2f%%)' % (needed_percent_of_overlap,
-                                           percent_of_overlap))
-        if (needed_percent_of_overlap > 0 and
-                (right_boundary_ts == left_boundary_ts or
-                 (right_boundary_ts is None
-                  and maybe_next_timestamp_is_left_boundary))):
-            LOG.debug("We didn't find points that overlap in those "
-                      "timeseries. "
-                      "right_boundary_ts=%(right_boundary_ts)s, "
-                      "left_boundary_ts=%(left_boundary_ts)s, "
-                      "groups=%(groups)s" % {
-                          'right_boundary_ts': right_boundary_ts,
-                          'left_boundary_ts': left_boundary_ts,
-                          'groups': list(grouped)
-                      })
-            raise UnAggregableTimeseries('No overlap')
+            if to_timestamp is not None or from_timestamp is not None:
+                maximum = len(grouped)
+                percent_of_overlap = (float(maximum - holes) * 100.0 /
+                                      float(maximum))
+                if percent_of_overlap < needed_percent_of_overlap:
+                    raise UnAggregableTimeseries(
+                        'Less than %f%% of datapoints overlap in this '
+                        'timespan (%.2f%%)' % (needed_percent_of_overlap,
+                                               percent_of_overlap))
+            if (needed_percent_of_overlap > 0 and
+                    (right_boundary_ts == left_boundary_ts or
+                     (right_boundary_ts is None
+                      and maybe_next_timestamp_is_left_boundary))):
+                LOG.debug("We didn't find points that overlap in those "
+                          "timeseries. "
+                          "right_boundary_ts=%(right_boundary_ts)s, "
+                          "left_boundary_ts=%(left_boundary_ts)s, "
+                          "groups=%(groups)s", {
+                              'right_boundary_ts': right_boundary_ts,
+                              'left_boundary_ts': left_boundary_ts,
+                              'groups': list(grouped)
+                          })
+                raise UnAggregableTimeseries('No overlap')
 
         # NOTE(sileht): this call the aggregation method on already
         # aggregated values, for some kind of aggregation this can
