@@ -1,5 +1,5 @@
 # Copyright (c) 2013 Mirantis Inc.
-# Copyright (c) 2015 Red Hat
+# Copyright (c) 2015-2017 Red Hat
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,32 +14,39 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import multiprocessing
+import sys
 import threading
 import time
-import uuid
 
-from concurrent import futures
 import cotyledon
+from cotyledon import oslo_config_glue
 from futurist import periodics
 import msgpack
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
-import retrying
 import six
+import tenacity
 import tooz
 from tooz import coordination
 
 from gnocchi import archive_policy
+from gnocchi import genconfig
 from gnocchi import indexer
 from gnocchi import service
 from gnocchi import statsd as statsd_service
 from gnocchi import storage
+from gnocchi.storage import incoming
+from gnocchi import utils
 
 
 LOG = log.getLogger(__name__)
 
 
+def config_generator():
+    return genconfig.prehook(None, sys.argv[1:])
+
+
 def upgrade():
     conf = cfg.ConfigOpts()
     conf.register_cli_opts([
@@ -56,12 +63,12 @@ def upgrade():
     index = indexer.get_driver(conf)
     index.connect()
     if not conf.skip_index:
-        LOG.info("Upgrading indexer %s" % index)
+        LOG.info("Upgrading indexer %s", index)
         index.upgrade(
             create_legacy_resource_types=conf.create_legacy_resource_types)
     if not conf.skip_storage:
         s = storage.get_driver(conf)
-        LOG.info("Upgrading storage %s" % s)
+        LOG.info("Upgrading storage %s", s)
         s.upgrade(index)
 
     if (not conf.skip_archive_policies_creation
@@ -76,14 +83,6 @@ def statsd():
     statsd_service.start()
 
 
-class Retry(Exception):
-    pass
-
-
-def retry_if_retry_is_raised(exception):
-    return isinstance(exception, Retry)
-
-
 class MetricProcessBase(cotyledon.Service):
     def __init__(self, worker_id, conf, interval_delay=0):
         super(MetricProcessBase, self).__init__(worker_id)
@@ -93,22 +92,10 @@ class MetricProcessBase(cotyledon.Service):
         self._shutdown = threading.Event()
         self._shutdown_done = threading.Event()
 
-    # Retry with exponential backoff for up to 1 minute
-    @retrying.retry(wait_exponential_multiplier=500,
-                    wait_exponential_max=60000,
-                    retry_on_exception=retry_if_retry_is_raised)
     def _configure(self):
-        try:
-            self.store = storage.get_driver(self.conf)
-        except storage.StorageError as e:
-            LOG.error("Unable to initialize storage: %s" % e)
-            raise Retry(e)
-        try:
-            self.index = indexer.get_driver(self.conf)
-            self.index.connect()
-        except indexer.IndexerException as e:
-            LOG.error("Unable to initialize indexer: %s" % e)
-            raise Retry(e)
+        self.store = storage.get_driver(self.conf)
+        self.index = indexer.get_driver(self.conf)
+        self.index.connect()
 
     def run(self):
         self._configure()
@@ -130,7 +117,7 @@ class MetricProcessBase(cotyledon.Service):
 
     @staticmethod
     def close_services():
-        raise NotImplementedError
+        pass
 
     @staticmethod
     def _run_job():
@@ -142,15 +129,18 @@ class MetricReporting(MetricProcessBase):
 
     def __init__(self, worker_id, conf):
         super(MetricReporting, self).__init__(
-            worker_id, conf, conf.storage.metric_reporting_delay)
+            worker_id, conf, conf.metricd.metric_reporting_delay)
 
     def _run_job(self):
         try:
-            report = self.store.measures_report(details=False)
+            report = self.store.incoming.measures_report(details=False)
             LOG.info("%d measurements bundles across %d "
                      "metrics wait to be processed.",
                      report['summary']['measures'],
                      report['summary']['metrics'])
+        except incoming.ReportGenerationError:
+            LOG.warning("Unable to compute backlog. Retrying at next "
+                        "interval.")
         except Exception:
             LOG.error("Unexpected error during pending measures reporting",
                       exc_info=True)
@@ -162,17 +152,13 @@ class MetricScheduler(MetricProcessBase):
     GROUP_ID = "gnocchi-scheduler"
     SYNC_RATE = 30
     TASKS_PER_WORKER = 16
-
-    def _enable_coordination(self, conf):
-        self._coord = coordination.get_coordinator(
-            conf.storage.coordination_url, self._my_id)
-        self._coord.start(start_heart=True)
+    BLOCK_SIZE = 4
 
     def __init__(self, worker_id, conf, queue):
         super(MetricScheduler, self).__init__(
-            worker_id, conf, conf.storage.metric_processing_delay)
-        self._my_id = str(uuid.uuid4())
-        self._enable_coordination(conf)
+            worker_id, conf, conf.metricd.metric_processing_delay)
+        self._coord, self._my_id = utils.get_coordinator_and_start(
+            conf.storage.coordination_url)
         self.queue = queue
         self.previously_scheduled_metrics = set()
         self.workers = conf.metricd.workers
@@ -200,10 +186,7 @@ class MetricScheduler(MetricProcessBase):
             self.block_index = 0
             self.block_size = self.block_size_default
 
-    # Retry with exponential backoff for up to 1 minute
-    @retrying.retry(wait_exponential_multiplier=500,
-                    wait_exponential_max=60000,
-                    retry_on_exception=retry_if_retry_is_raised)
+    @utils.retry
     def _configure(self):
         super(MetricScheduler, self)._configure()
         try:
@@ -217,9 +200,7 @@ class MetricScheduler(MetricProcessBase):
             def run_watchers():
                 self._coord.run_watchers()
 
-            self.periodic = periodics.PeriodicWorker.create(
-                [], executor_factory=lambda:
-                futures.ThreadPoolExecutor(max_workers=10))
+            self.periodic = periodics.PeriodicWorker.create([])
             self.periodic.add(run_watchers)
             t = threading.Thread(target=self.periodic.start)
             t.daemon = True
@@ -233,7 +214,7 @@ class MetricScheduler(MetricProcessBase):
                 create_group_req.get()
             except coordination.GroupAlreadyExist:
                 pass
-            raise Retry(e)
+            raise tenacity.TryAgain(e)
         except tooz.NotImplemented:
             LOG.warning('Configured coordination driver does not support '
                         'required functionality. Coordination is disabled.')
@@ -243,8 +224,9 @@ class MetricScheduler(MetricProcessBase):
 
     def _run_job(self):
         try:
-            metrics = set(self.store.list_metric_with_measures_to_process(
-                self.block_size, self.block_index))
+            metrics = set(
+                self.store.incoming.list_metric_with_measures_to_process(
+                    self.block_size, self.block_index))
             if metrics and not self.queue.empty():
                 # NOTE(gordc): drop metrics we previously process to avoid
                 #              handling twice
@@ -255,9 +237,10 @@ class MetricScheduler(MetricProcessBase):
                     LOG.warning('Metric processing lagging scheduling rate. '
                                 'It is recommended to increase the number of '
                                 'workers or to lengthen processing interval.')
-            for m_id in metrics:
-                self.queue.put(m_id)
-            self.previously_scheduled_metrics = metrics
+            metrics = list(metrics)
+            for i in six.moves.range(0, len(metrics), self.BLOCK_SIZE):
+                self.queue.put(metrics[i:i + self.BLOCK_SIZE])
+            self.previously_scheduled_metrics = set(metrics)
             LOG.debug("%d metrics scheduled for processing.", len(metrics))
         except Exception:
             LOG.error("Unexpected error scheduling metrics for processing",
@@ -269,7 +252,6 @@ class MetricScheduler(MetricProcessBase):
             self.periodic.wait()
         self._coord.leave_group(self.GROUP_ID)
         self._coord.stop()
-        self.queue.close()
 
 
 class MetricJanitor(MetricProcessBase):
@@ -277,7 +259,7 @@ class MetricJanitor(MetricProcessBase):
 
     def __init__(self,  worker_id, conf):
         super(MetricJanitor, self).__init__(
-            worker_id, conf, conf.storage.metric_cleanup_delay)
+            worker_id, conf, conf.metricd.metric_cleanup_delay)
 
     def _run_job(self):
         try:
@@ -289,44 +271,81 @@ class MetricJanitor(MetricProcessBase):
 
 class MetricProcessor(MetricProcessBase):
     name = "processing"
-    BLOCK_SIZE = 4
 
     def __init__(self, worker_id, conf, queue):
-        super(MetricProcessor, self).__init__(worker_id, conf, 1)
+        super(MetricProcessor, self).__init__(worker_id, conf, 0)
         self.queue = queue
 
     def _run_job(self):
         try:
-            metrics = []
-            while len(metrics) < self.BLOCK_SIZE:
-                try:
-                    metrics.append(self.queue.get(block=False))
-                except six.moves.queue.Empty:
-                    # queue might be emptied by other workers, continue on.
-                    break
-            if metrics:
-                self.store.process_background_tasks(self.index, metrics)
+            try:
+                metrics = self.queue.get(block=True, timeout=10)
+            except six.moves.queue.Empty:
+                # NOTE(sileht): Allow the process to exit gracefully every
+                # 10 seconds
+                return
+            self.store.process_background_tasks(self.index, metrics)
         except Exception:
             LOG.error("Unexpected error during measures processing",
                       exc_info=True)
 
-    def close_services(self):
-        self.queue.close()
-
 
 class MetricdServiceManager(cotyledon.ServiceManager):
     def __init__(self, conf):
         super(MetricdServiceManager, self).__init__()
+        oslo_config_glue.setup(self, conf)
+
         self.conf = conf
         self.queue = multiprocessing.Manager().Queue()
 
         self.add(MetricScheduler, args=(self.conf, self.queue))
-        self.add(MetricProcessor, args=(self.conf, self.queue),
-                 workers=conf.metricd.workers)
-        self.add(MetricReporting, args=(self.conf,))
+        self.metric_processor_id = self.add(
+            MetricProcessor, args=(self.conf, self.queue),
+            workers=conf.metricd.workers)
+        if self.conf.metricd.metric_reporting_delay >= 0:
+            self.add(MetricReporting, args=(self.conf,))
         self.add(MetricJanitor, args=(self.conf,))
 
+        self.register_hooks(on_reload=self.on_reload)
+
+    def on_reload(self):
+        # NOTE(sileht): We do not implement reload() in Workers so all workers
+        # will received SIGHUP and exit gracefully, then their will be
+        # restarted with the new number of workers. This is important because
+        # we use the number of worker to declare the capability in tooz and
+        # to select the block of metrics to proceed.
+        self.reconfigure(self.metric_processor_id,
+                         workers=self.conf.metricd.workers)
+
+    def run(self):
+        super(MetricdServiceManager, self).run()
+        self.queue.close()
+
+
+def metricd_tester(conf):
+    # NOTE(sileht): This method is designed to be profiled, we
+    # want to avoid issues with profiler and os.fork(), that
+    # why we don't use the MetricdServiceManager.
+    index = indexer.get_driver(conf)
+    index.connect()
+    s = storage.get_driver(conf)
+    metrics = s.incoming.list_metric_with_measures_to_process(
+        conf.stop_after_processing_metrics, 0)
+    s.process_new_measures(index, metrics, True)
+
 
 def metricd():
-    conf = service.prepare_service()
-    MetricdServiceManager(conf).run()
+    conf = cfg.ConfigOpts()
+    conf.register_cli_opts([
+        cfg.IntOpt("stop-after-processing-metrics",
+                   default=0,
+                   min=0,
+                   help="Number of metrics to process without workers, "
+                   "for testing purpose"),
+    ])
+    conf = service.prepare_service(conf=conf)
+
+    if conf.stop_after_processing_metrics:
+        metricd_tester(conf)
+    else:
+        MetricdServiceManager(conf).run()
