@@ -1,6 +1,5 @@
 # -*- encoding: utf-8 -*-
 #
-# Copyright © 2017 Red Hat, Inc.
 # Copyright © 2014-2015 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -27,23 +26,6 @@ OPTS = [
     cfg.StrOpt('driver',
                default='file',
                help='Storage driver to use'),
-    cfg.IntOpt('metric_processing_delay',
-               default=30,
-               help="How many seconds to wait between "
-               "scheduling new metrics to process"),
-    cfg.IntOpt('tasks_per_worker',
-               default=256,
-               min=1,
-               help="How many tasks to assign each metricd worker when "
-               "scheduling measures processing jobs"),
-    cfg.IntOpt('metric_reporting_delay',
-               default=120,
-               help="How many seconds to wait between "
-               "metric ingestion reporting"),
-    cfg.IntOpt('metric_cleanup_delay',
-               default=300,
-               help="How many seconds to wait between "
-               "cleaning of expired data"),
 ]
 
 LOG = log.getLogger(__name__)
@@ -62,14 +44,12 @@ class Measure(object):
 
 class Metric(object):
     def __init__(self, id, archive_policy,
-                 created_by_user_id=None,
-                 created_by_project_id=None,
+                 creator=None,
                  name=None,
                  resource_id=None):
         self.id = id
         self.archive_policy = archive_policy
-        self.created_by_user_id = created_by_user_id
-        self.created_by_project_id = created_by_project_id
+        self.creator = creator
         self.name = name
         self.resource_id = resource_id
 
@@ -83,8 +63,7 @@ class Metric(object):
         return (isinstance(other, Metric)
                 and self.id == other.id
                 and self.archive_policy == other.archive_policy
-                and self.created_by_user_id == other.created_by_user_id
-                and self.created_by_project_id == other.created_by_project_id
+                and self.creator == other.creator
                 and self.name == other.name
                 and self.resource_id == other.resource_id)
 
@@ -150,32 +129,33 @@ class MetricUnaggregatable(StorageError):
             % (", ".join((str(m.id) for m in metrics)), reason))
 
 
-def get_driver_class(conf):
+def get_driver_class(namespace, conf):
     """Return the storage driver class.
 
     :param conf: The conf to use to determine the driver.
     """
-    return driver.DriverManager('gnocchi.storage',
-                                conf.storage.driver).driver
+    return driver.DriverManager(namespace,
+                                conf.driver).driver
 
 
 def get_driver(conf):
     """Return the configured driver."""
-    return get_driver_class(conf)(conf.storage)
+    incoming = get_driver_class('gnocchi.incoming', conf.incoming)(
+        conf.incoming)
+    return get_driver_class('gnocchi.storage', conf.storage)(
+        conf.storage, incoming)
 
 
 class StorageDriver(object):
-    @staticmethod
-    def __init__(conf):
-        pass
+    def __init__(self, conf, incoming):
+        self.incoming = incoming
 
     @staticmethod
     def stop():
         pass
 
-    @staticmethod
-    def upgrade(index):
-        pass
+    def upgrade(self, index):
+        self.incoming.upgrade(index)
 
     def process_background_tasks(self, index, metrics, sync=False):
         """Process background tasks for this storage.
@@ -213,7 +193,7 @@ class StorageDriver(object):
             except Exception:
                 if sync:
                     raise
-                LOG.error("Unable to expunge metric %s from storage" % m,
+                LOG.error("Unable to expunge metric %s from storage", m,
                           exc_info=True)
                 continue
             try:
@@ -223,23 +203,6 @@ class StorageDriver(object):
                 # time, not a big deal
                 pass
 
-    def add_measures(self, metric, measures):
-        """Add a measure to a metric.
-
-        :param metric: The metric measured.
-        :param measures: The actual measures.
-        """
-        self.add_measures_batch({metric: measures})
-
-    @staticmethod
-    def add_measures_batch(metrics_and_measures):
-        """Add a batch of measures for some metrics.
-
-        :param metrics_and_measures: A dict where keys
-        are metrics and value are measure.
-        """
-        raise exceptions.NotImplementedError
-
     @staticmethod
     def process_new_measures(indexer, metrics, sync=False):
         """Process added measures in background.
@@ -248,20 +211,9 @@ class StorageDriver(object):
         the measures sent to metrics. This is used for that.
         """
 
-    @staticmethod
-    def measures_report(details=True):
-        """Return a report of pending to process measures.
-
-        Only useful for drivers that process measurements in background
-
-        :return: {'summary': {'metrics': count, 'measures': count},
-                  'details': {metric_id: pending_measures_count}}
-        """
-        raise exceptions.NotImplementedError
-
     @staticmethod
     def get_measures(metric, from_timestamp=None, to_timestamp=None,
-                     aggregation='mean', granularity=None):
+                     aggregation='mean', granularity=None, resample=None):
         """Get a measure to a metric.
 
         :param metric: The metric measured.
@@ -269,6 +221,7 @@ class StorageDriver(object):
         :param to timestamp: The timestamp to get the measure to.
         :param aggregation: The type of aggregation to retrieve.
         :param granularity: The granularity to retrieve.
+        :param resample: The granularity to resample to.
         """
         if aggregation not in metric.archive_policy.aggregation_methods:
             raise AggregationDoesNotExist(metric, aggregation)
@@ -280,9 +233,9 @@ class StorageDriver(object):
     @staticmethod
     def get_cross_metric_measures(metrics, from_timestamp=None,
                                   to_timestamp=None, aggregation='mean',
-                                  reaggregation=None,
-                                  granularity=None,
-                                  needed_overlap=None):
+                                  reaggregation=None, resample=None,
+                                  granularity=None, needed_overlap=None,
+                                  fill=None):
         """Get aggregated measures of multiple entities.
 
         :param entities: The entities measured to aggregate.
@@ -292,6 +245,8 @@ class StorageDriver(object):
         :param aggregation: The type of aggregation to retrieve.
         :param reaggregation: The type of aggregation to compute
                               on the retrieved measures.
+        :param resample: The granularity to resample to.
+        :param fill: The value to use to fill in missing data in series.
         """
         for metric in metrics:
             if aggregation not in metric.archive_policy.aggregation_methods:
@@ -304,7 +259,8 @@ class StorageDriver(object):
     @staticmethod
     def search_value(metrics, query, from_timestamp=None,
                      to_timestamp=None,
-                     aggregation='mean'):
+                     aggregation='mean',
+                     granularity=None):
         """Search for an aggregated value that realizes a predicate.
 
         :param metrics: The list of metrics to look into.
@@ -312,6 +268,7 @@ class StorageDriver(object):
         :param from_timestamp: The timestamp to get the measure from.
         :param to_timestamp: The timestamp to get the measure to.
         :param aggregation: The type of aggregation to retrieve.
+        :param granularity: The granularity to retrieve.
         """
         raise exceptions.NotImplementedError
 
