@@ -14,6 +14,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import base64
 import calendar
 import contextlib
 import datetime
@@ -32,6 +33,7 @@ from testtools import testcase
 import webtest
 
 from gnocchi import archive_policy
+from gnocchi import rest
 from gnocchi.rest import app
 from gnocchi.tests import base as tests_base
 from gnocchi import utils
@@ -53,29 +55,43 @@ class TestingApp(webtest.TestApp):
     USER_ID_2 = str(uuid.uuid4())
     PROJECT_ID_2 = str(uuid.uuid4())
 
+    INVALID_TOKEN = str(uuid.uuid4())
+
     def __init__(self, *args, **kwargs):
-        self.auth = kwargs.pop('auth')
+        self.auth_mode = kwargs.pop('auth_mode')
         self.storage = kwargs.pop('storage')
         self.indexer = kwargs.pop('indexer')
         super(TestingApp, self).__init__(*args, **kwargs)
         # Setup Keystone auth_token fake cache
         self.token = self.VALID_TOKEN
+        # Setup default user for basic auth
+        self.user = self.USER_ID.encode('ascii')
 
     @contextlib.contextmanager
     def use_admin_user(self):
-        if not self.auth:
-            raise testcase.TestSkipped("No auth enabled")
-        old_token = self.token
-        self.token = self.VALID_TOKEN_ADMIN
-        try:
-            yield
-        finally:
-            self.token = old_token
+        if self.auth_mode == "keystone":
+            old_token = self.token
+            self.token = self.VALID_TOKEN_ADMIN
+            try:
+                yield
+            finally:
+                self.token = old_token
+        elif self.auth_mode == "basic":
+            old_user = self.user
+            self.user = b"admin"
+            try:
+                yield
+            finally:
+                self.user = old_user
+        elif self.auth_mode == "noauth":
+            raise testcase.TestSkipped("auth mode is noauth")
+        else:
+            raise RuntimeError("Unknown auth_mode")
 
     @contextlib.contextmanager
     def use_another_user(self):
-        if not self.auth:
-            raise testcase.TestSkipped("No auth enabled")
+        if self.auth_mode != "keystone":
+            raise testcase.TestSkipped("Auth mode is not Keystone")
         old_token = self.token
         self.token = self.VALID_TOKEN_2
         try:
@@ -83,11 +99,30 @@ class TestingApp(webtest.TestApp):
         finally:
             self.token = old_token
 
+    @contextlib.contextmanager
+    def use_invalid_token(self):
+        if self.auth_mode != "keystone":
+            raise testcase.TestSkipped("Auth mode is not Keystone")
+        old_token = self.token
+        self.token = self.INVALID_TOKEN
+        try:
+            yield
+        finally:
+            self.token = old_token
+
     def do_request(self, req, *args, **kwargs):
-        if self.auth:
-            req.headers['X-Auth-Token'] = self.token
+        if self.auth_mode in "keystone":
+            if self.token is not None:
+                req.headers['X-Auth-Token'] = self.token
+        elif self.auth_mode == "basic":
+            req.headers['Authorization'] = (
+                b"basic " + base64.b64encode(self.user + b":")
+            )
+        elif self.auth_mode == "noauth":
+            req.headers['X-User-Id'] = self.USER_ID
+            req.headers['X-Project-Id'] = self.PROJECT_ID
         response = super(TestingApp, self).do_request(req, *args, **kwargs)
-        metrics = self.storage.list_metric_with_measures_to_process(
+        metrics = self.storage.incoming.list_metric_with_measures_to_process(
             None, None, full=True)
         self.storage.process_background_tasks(self.indexer, metrics, sync=True)
         return response
@@ -96,49 +131,48 @@ class TestingApp(webtest.TestApp):
 class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
 
     scenarios = [
-        ('noauth', dict(auth=False)),
-        ('keystone', dict(auth=True)),
+        ('basic', dict(auth_mode="basic")),
+        ('keystone', dict(auth_mode="keystone")),
+        ('noauth', dict(auth_mode="noauth")),
     ]
 
     def setUp(self):
         super(RestTest, self).setUp()
-        self.conf.set_override('paste_config',
-                               self.path_get('etc/gnocchi/api-paste.ini'),
-                               group="api")
 
-        self.auth_token_fixture = self.useFixture(
-            ksm_fixture.AuthTokenFixture())
-        self.auth_token_fixture.add_token_data(
-            is_v2=True,
-            token_id=TestingApp.VALID_TOKEN_ADMIN,
-            user_id=TestingApp.USER_ID_ADMIN,
-            user_name='adminusername',
-            project_id=TestingApp.PROJECT_ID_ADMIN,
-            role_list=['admin'])
-        self.auth_token_fixture.add_token_data(
-            is_v2=True,
-            token_id=TestingApp.VALID_TOKEN,
-            user_id=TestingApp.USER_ID,
-            user_name='myusername',
-            project_id=TestingApp.PROJECT_ID,
-            role_list=["member"])
-        self.auth_token_fixture.add_token_data(
-            is_v2=True,
-            token_id=TestingApp.VALID_TOKEN_2,
-            user_id=TestingApp.USER_ID_2,
-            user_name='myusername2',
-            project_id=TestingApp.PROJECT_ID_2,
-            role_list=["member"])
+        if self.auth_mode == "keystone":
+            self.auth_token_fixture = self.useFixture(
+                ksm_fixture.AuthTokenFixture())
+            self.auth_token_fixture.add_token_data(
+                is_v2=True,
+                token_id=TestingApp.VALID_TOKEN_ADMIN,
+                user_id=TestingApp.USER_ID_ADMIN,
+                user_name='adminusername',
+                project_id=TestingApp.PROJECT_ID_ADMIN,
+                role_list=['admin'])
+            self.auth_token_fixture.add_token_data(
+                is_v2=True,
+                token_id=TestingApp.VALID_TOKEN,
+                user_id=TestingApp.USER_ID,
+                user_name='myusername',
+                project_id=TestingApp.PROJECT_ID,
+                role_list=["member"])
+            self.auth_token_fixture.add_token_data(
+                is_v2=True,
+                token_id=TestingApp.VALID_TOKEN_2,
+                user_id=TestingApp.USER_ID_2,
+                user_name='myusername2',
+                project_id=TestingApp.PROJECT_ID_2,
+                role_list=["member"])
+
+        self.conf.set_override("auth_mode", self.auth_mode, group="api")
 
         self.app = TestingApp(app.load_app(conf=self.conf,
-                                           appname="gnocchi+auth"
-                                           if self.auth else "gnocchi+noauth",
                                            indexer=self.index,
                                            storage=self.storage,
                                            not_implemented_middleware=False),
                               storage=self.storage,
                               indexer=self.index,
-                              auth=self.auth)
+                              auth_mode=self.auth_mode)
 
     # NOTE(jd) Used at least by docs
     @staticmethod
@@ -175,11 +209,9 @@ class RootTest(RestTest):
         with self.app.use_admin_user():
             r = self.app.get("/v1/status")
         status = json.loads(r.text)
-        # We are sure this is empty because we call process_measures() each
-        # time we do a REST request in this TestingApp.
-        self.assertEqual({}, status['storage']['measures_to_process'])
-        self.assertIs(type(status['storage']['summary']['metrics']), int)
-        self.assertIs(type(status['storage']['summary']['measures']), int)
+        self.assertIsInstance(status['storage']['measures_to_process'], dict)
+        self.assertIsInstance(status['storage']['summary']['metrics'], int)
+        self.assertIsInstance(status['storage']['summary']['measures'], int)
 
 
 class ArchivePolicyTest(RestTest):
@@ -385,10 +417,10 @@ class MetricTest(RestTest):
             self.app.get("/v1/metric/%s/measures" % metric['id'],
                          status=403)
 
-    @mock.patch.object(utils, 'utcnow')
+    @mock.patch.object(timeutils, 'utcnow')
     def test_get_measure_start_relative(self, utcnow):
         """Make sure the timestamps can be relative to now."""
-        utcnow.return_value = utils.datetime_utc(2014, 1, 1, 10, 23)
+        utcnow.return_value = datetime.datetime(2014, 1, 1, 10, 23)
         result = self.app.post_json("/v1/metric",
                                     params={"archive_policy_name": "high"})
         metric = json.loads(result.text)
@@ -400,7 +432,7 @@ class MetricTest(RestTest):
             % metric['id'],
             status=200)
         result = json.loads(ret.text)
-        now = utils.utcnow()
+        now = utils.datetime_utc(2014, 1, 1, 10, 23)
         self.assertEqual([
             ['2014-01-01T10:00:00+00:00', 3600.0, 1234.2],
             [(now
@@ -607,12 +639,15 @@ class ResourceTest(RestTest):
         self.resource = self.attributes.copy()
         # Set original_resource_id
         self.resource['original_resource_id'] = self.resource['id']
-        if self.auth:
-            self.resource['created_by_user_id'] = TestingApp.USER_ID
+        self.resource['created_by_user_id'] = TestingApp.USER_ID
+        if self.auth_mode in ("keystone", "noauth"):
             self.resource['created_by_project_id'] = TestingApp.PROJECT_ID
-        else:
-            self.resource['created_by_user_id'] = None
-            self.resource['created_by_project_id'] = None
+            self.resource['creator'] = (
+                TestingApp.USER_ID + ":" + TestingApp.PROJECT_ID
+            )
+        elif self.auth_mode == "basic":
+            self.resource['created_by_project_id'] = ""
+            self.resource['creator'] = TestingApp.USER_ID
         self.resource['ended_at'] = None
         self.resource['metrics'] = {}
         if 'user_id' not in self.resource:
@@ -686,16 +721,6 @@ class ResourceTest(RestTest):
         self.assertIn("Resource %s already exists" % self.attributes['id'],
                       result.text)
 
-    def test_post_unix_timestamp(self):
-        self.attributes['started_at'] = "1400580045.856219"
-        result = self.app.post_json(
-            "/v1/resource/" + self.resource_type,
-            params=self.attributes,
-            status=201)
-        resource = json.loads(result.text)
-        self.assertEqual(u"2014-05-20T10:00:45.856219+00:00",
-                         resource['started_at'])
-
     def test_post_invalid_timestamp(self):
         self.attributes['started_at'] = "2014-01-01 02:02:02"
         self.attributes['ended_at'] = "2013-01-01 02:02:02"
@@ -1369,7 +1394,7 @@ class ResourceTest(RestTest):
             "/v1/search/resource/" + self.resource_type,
             params={"=": {"foobar": "baz"}},
             status=400)
-        self.assertIn("Resource " + self.resource_type
+        self.assertIn("Resource type " + self.resource_type
                       + " has no foobar attribute",
                       result.text)
 
@@ -1820,3 +1845,73 @@ class GenericResourceTest(RestTest):
             "Invalid input: extra keys not allowed @ data["
             + repr(u'wrongoperator') + "]",
             result.text)
+
+
+class QueryStringSearchAttrFilterTest(tests_base.TestCase):
+    def _do_test(self, expr, expected):
+        req = rest.QueryStringSearchAttrFilter.parse(expr)
+        self.assertEqual(expected, req)
+
+    def test_search_query_builder(self):
+        self._do_test('foo=7EED6CC3-EDC8-48C9-8EF6-8A36B9ACC91C',
+                      {"=": {"foo": "7EED6CC3-EDC8-48C9-8EF6-8A36B9ACC91C"}})
+        self._do_test('foo=7EED6CC3EDC848C98EF68A36B9ACC91C',
+                      {"=": {"foo": "7EED6CC3EDC848C98EF68A36B9ACC91C"}})
+        self._do_test('foo=bar', {"=": {"foo": "bar"}})
+        self._do_test('foo!=1', {"!=": {"foo": 1.0}})
+        self._do_test('foo=True', {"=": {"foo": True}})
+        self._do_test('foo=null', {"=": {"foo": None}})
+        self._do_test('foo="null"', {"=": {"foo": "null"}})
+        self._do_test('foo in ["null", "foo"]',
+                      {"in": {"foo": ["null", "foo"]}})
+        self._do_test(u'foo="quote" and bar≠1',
+                      {"and": [{u"≠": {"bar": 1}},
+                               {"=": {"foo": "quote"}}]})
+        self._do_test('foo="quote" or bar like "%%foo"',
+                      {"or": [{"like": {"bar": "%%foo"}},
+                              {"=": {"foo": "quote"}}]})
+
+        self._do_test('not (foo="quote" or bar like "%%foo" or foo="what!" '
+                      'or bar="who?")',
+                      {"not": {"or": [
+                          {"=": {"bar": "who?"}},
+                          {"=": {"foo": "what!"}},
+                          {"like": {"bar": "%%foo"}},
+                          {"=": {"foo": "quote"}},
+                      ]}})
+
+        self._do_test('(foo="quote" or bar like "%%foo" or not foo="what!" '
+                      'or bar="who?") and cat="meme"',
+                      {"and": [
+                          {"=": {"cat": "meme"}},
+                          {"or": [
+                              {"=": {"bar": "who?"}},
+                              {"not": {"=": {"foo": "what!"}}},
+                              {"like": {"bar": "%%foo"}},
+                              {"=": {"foo": "quote"}},
+                          ]}
+                      ]})
+
+        self._do_test('foo="quote" or bar like "%%foo" or foo="what!" '
+                      'or bar="who?" and cat="meme"',
+                      {"or": [
+                          {"and": [
+                              {"=": {"cat": "meme"}},
+                              {"=": {"bar": "who?"}},
+                          ]},
+                          {"=": {"foo": "what!"}},
+                          {"like": {"bar": "%%foo"}},
+                          {"=": {"foo": "quote"}},
+                      ]})
+
+        self._do_test('foo="quote" or bar like "%%foo" and foo="what!" '
+                      'or bar="who?" or cat="meme"',
+                      {"or": [
+                          {"=": {"cat": "meme"}},
+                          {"=": {"bar": "who?"}},
+                          {"and": [
+                              {"=": {"foo": "what!"}},
+                              {"like": {"bar": "%%foo"}},
+                          ]},
+                          {"=": {"foo": "quote"}},
+                      ]})
